from werkzeug.exceptions import BadRequest, NotFound
from sqlalchemy.exc import IntegrityError

from flaschengeist import logger
from flaschengeist.database import db
from flaschengeist.plugins.schedule.models import EventType, Event, EventSlot, JobSlot, JobType, Job


def update():
    db.session.commit()


def get_event_types():
    return EventType.query.all()


def get_event_type(name):
    et = EventType.query.filter(EventType.name == name).one_or_none()
    if not et:
        raise NotFound
    return et


def create_event_type(name):
    try:
        event = EventType(name=name)
        db.session.add(event)
        db.session.commit()
        return event
    except IntegrityError:
        raise BadRequest("Name already exists")


def rename_event_type(name, new_name):
    event_type = get_event_type(name)
    event_type.name = new_name
    try:
        db.session.commit()
    except IntegrityError:
        raise BadRequest("Name already exists")


def delete_event_type(name):
    event_type = get_event_type(name)
    db.session.delete(event_type)
    try:
        db.session.commit()
    except IntegrityError:
        raise BadRequest("Type still in use")


def get_job_types():
    return JobType.query.all()


def get_job_type(type_id):
    job_type = JobType.query.get(type_id)
    if not job_type:
        raise NotFound
    return job_type


def create_job_type(name):
    try:
        job_type = JobType(name=name)
        db.session.add(job_type)
        db.session.commit()
        return job_type
    except IntegrityError:
        raise BadRequest("Name already exists")


def rename_job_type(name, new_name):
    job_type = get_job_type(name)
    job_type.name = new_name
    try:
        db.session.commit()
    except IntegrityError:
        raise BadRequest("Name already exists")


def delete_job_type(name):
    job_type = get_job_type(name)
    db.session.delete(job_type)
    try:
        db.session.commit()
    except IntegrityError:
        raise BadRequest("Type still in use")


def get_event(event_id):
    event = Event.query.get(event_id)
    if event is None:
        raise NotFound
    return event


def get_events(start, end):
    """Query events which start from begin until end
    Args:
        start (datetime): Earliest start
        end (datetime): Latest start

    Returns: collection of Event objects
    """
    return Event.query.filter((start <= Event.start), (Event.start < end)).all()


def delete_event(event_id):
    """Delete event with given ID
    Args:
        event_id: id of Event to delete

    Raises:
        NotFound if not found
    """
    event = get_event(event_id)
    db.session.delete(event)
    db.session.commit()


def create_event(event_type, start, slots=[], description=None):
    try:
        logger.debug(event_type)
        event = Event(start=start, description=description, type=event_type, slots=slots)
        db.session.add(event)
        db.session.commit()
        return event
    except IntegrityError:
        logger.debug("Database error when creating new event", exc_info=True)
        raise BadRequest


def get_event_slot(slot_id):
    slot = EventSlot.query.get(slot_id)
    if slot is None:
        raise NotFound
    return slot


def add_event_slot(event, start, end=None):
    event_slot = EventSlot(start=start, end=end, event_=event)
    if start < event.start:
        raise BadRequest("Start before event start")
    db.session.add(event_slot)
    db.session.commit()
    return event_slot


def remove_event_slot(event, slot_id):
    slot = get_event_slot(slot_id)
    if slot in event.slots:
        event.slots.remove(slot)
    else:
        raise NotFound
    db.session.commit()


def get_job_slot(event_slot_id, job_type):
    jt = (
        JobSlot.query.filter(JobSlot._type_id == job_type).filter(JobSlot._event_slot_id == event_slot_id).one_or_none()
    )
    if jt is None:
        raise NotFound
    return jt


def add_job_slot(event_slot, job_type, required_jobs):
    job_slot = JobSlot(type=job_type, required_jobs=required_jobs, event_slot_=event_slot)
    try:
        db.session.add(job_slot)
        db.session.commit()
    except IntegrityError:
        raise BadRequest("JobSlot with that type already exists on this EventSlot")


def delete_job_slot(job_slot):
    db.session.delete(job_slot)
    db.session.commit()


def assign_job(job_slot, user, value):
    job = Job.query.get((job_slot.id_, user._id))
    if job:
        job.value = value
    else:
        job = Job(user_=user, value=value, slot_=job_slot)
        db.session.add(job)
    db.session.commit()
